package PureMessage::HTTPD::Config;
# parses some rudimentary config settings and allows them
# to be updated.

# settings this package is able to modify: 
#
# ServerName 
# Listen 
# PidFile
#

use strict;
use PureMessage::Config qw(expand_filename);
use PureMessage::Version qw($PREFIX);
use ActiveState::File::Atomic;
use PureMessage::Observer;
use base qw(Exporter);

our @EXPORT_OK = qw( %config %httpd_config );
our %config;

our $httpd_config; # alias that doesn't clash with other %configs
*httpd_config =\%config;

my @valid_keys = qw( ServerName Listen PidFile );

sub refresh {
  my $main = _parse_file("$PREFIX/etc/manager/httpd2/main.conf");
  my $ssl  = _parse_file("$PREFIX/etc/manager/httpd2/ssl.conf");
  my $value = {%$main, %$ssl};
  %config = $value ? %$value : ();
  return $value;
}

# takes a hash of values to change
# will notify: httpd2.conf_change
sub update {
  my %new = @_;
  my $value = _modify_file("$PREFIX/etc/manager/httpd2/main.conf",%new);

  refresh() if $value;
  notify(%new) if $value;
  return $value ? 1 : 0;
}

sub notify {
  my $observers = new PureMessage::Observer('httpd');
  $observers->notify('conf_updated',@_);
}

sub _modify_file {
  my $file = shift;
  my %change = @_;
  my $changes = 0;
  my $fa = ActiveState::File::Atomic->new($file,writable =>1);
  my @lines = split("\n",$fa->slurp);
  # We are not going to parse full apache files with nested blocks and such,
  # the format of the main.conf file /must/ be flat. If it isn't it is an 
  # error and we cannot continue.
  for(my $ln = 0; $ln <= $#lines; $ln++) {
    die "Invalid structure encountered when parsing $file at line $ln.\n" if $lines[$ln] =~ /^\s*</;
    if ($lines[$ln] =~ /^\s*([\w]+)(\s.*)?$/) {
      my ($name,$extra) = ($1,$2);
      next unless grep { $name eq $_ } keys %change;
      $lines[$ln] = "$name $change{$name}";
      delete($change{$name});   
      $changes++;
    }
  } 
  foreach my $left_over (keys %change) {
    push(@lines,"$left_over $change{$left_over}");
    $changes++;
  }
  $fa->commit_string(join("\n",@lines)) if $changes;
  return $changes;
}

sub _parse_file {
  my $file = shift;
  my @ok_keys = ref($_[0]) eq 'ARRAY' ? @$_[0] : @valid_keys;
  my %result;
  my $fa = ActiveState::File::Atomic->new($file);
  my @lines = split("\n",$fa->slurp);
  $fa->close();
  for(my $ln = 0; $ln <= $#lines; $ln++) {
    if ($lines[$ln] =~ /^\s*([\w]+)\s\*:(.*)?$/  || $lines[$ln] =~ /^\s*([\w]+)(\s.*)?$/) {
        my ($name,$extra) = ($1,$2);
      next unless grep { $name =~ $_ } @ok_keys;
      $extra =~ s/\s//g;
      # We are only interested in the first record in the file
      $result{$name} = $extra if(not defined($result{$name})); 
    }
  } 

  return \%result;
}

refresh();


