package PureMessage::HTTPD::Service;
use strict;
use warnings;
use Carp;
use ActiveState::Handy qw(run);
use ActiveState::Unix::ProcInfo qw(proc_info);
use PureMessage::Config qw(expand_filename);
use PureMessage::HTTPD::Config qw( %httpd_config);
use PureMessage::Version qw($PREFIX);
use PureMessage::Service::Config qw( is_stale stale );

use base 'Exporter';
our @EXPORT_OK = qw(
    httpd_pid
    httpd_is_running
    httpd_start
    httpd_stop
    httpd_restart
    httpd_status
    httpd_print_status
);

sub httpd_pid{
    my $pid_file = expand_filename($httpd_config{PidFile});
    my $pid = 0;
    return $pid unless -e $pid_file;
    if(open(PF, '<', $pid_file)){
	$pid = <PF>;
	chomp($pid);
	close PF;
    }
    return $pid;
}

sub httpd_is_running{
    my $pid = httpd_pid();
    return 0 unless $pid;
    my $info = proc_info ('root_pid' => $pid);
    return 0 unless $info && $$info{$pid}; 
    return 1;
}

sub httpd_start{
    return 1 if httpd_is_running();

    my $cmd = expand_filename('bin/httpd2');
    croak('Web server program does not have permissions to run.')
	unless -x $cmd;

    # httpd will only return true if there's a config error, 
    # or something else it finds wrong, but
    # ***only before it forks into the BG***.
    my $rc = 1;
    eval {$rc = run("\@$cmd >/dev/null 2>&1"); };
    return 0 if ($rc || $@);

    # Poll for httpd to start.
    # httpd 1.3.x won't write a PID file if
    # it gets [crit] or higher errors on startup.
    my $count = 10;
    while ($count--) {
	sleep(1);
	if (httpd_is_running()) {
	    return 1;
	}
    }

    return 0;
}

sub httpd_stop{
    return 1 unless httpd_is_running();
    my $pid = httpd_pid();
    my $count = 30;
    if(kill(0, $pid)){
	kill('TERM', $pid);
        while($count and kill(0,$pid)) {
          $count--;  
          sleep(1);
        }
        return $count;
    }
    return 0;
}

sub httpd_restart{
    my $c = 1;
    if(httpd_is_running()){
	$c = httpd_stop();
    }
    return 0 unless $c;
    return httpd_start();
}

# determine if our config is stale or not
sub config_status {
   my $self = shift;
   
   my @stat = stat(expand_filename($httpd_config{PidFile}));
   my $run_time = $stat[9];
   
   my @files = glob("$PREFIX/etc/manager/httpd2/*.conf");
   push(@files,"$PREFIX/etc/manager/httpd2.conf");
   
   my $status = 'current';
   
   foreach my $file (@files) {
     @stat = stat($file);
     next unless @stat;
     if($stat[9] > $run_time) {
       $status = 'stale';
       last;
     }
   }
   my $is_stale_time = is_stale('httpd');
   $status = 'stale' if $is_stale_time  and $is_stale_time > $run_time; 
   return $status;
}


sub httpd_status{
	my %info = ();
    $info{'listen'} = $httpd_config{'Listen'};
	$info{'pid'}      = httpd_pid();
	$info{'running'}  = httpd_is_running();
	#$info{'https'}    = 'ssl'; # XXX
    $info{'servername'} = $httpd_config{'ServerName'};
	my $https         = 'http'; # XXX	
	$info{'url'}      = "$https://$info{'servername'}:$info{'listen'}";
    $info{'config'}   = config_status() if $info{'running'};
	return \%info;
}

sub httpd_print_status{
    my $info = httpd_status();
    unless ($$info{'running'}) {
	print "HTTPD is not running.\n";
	return 0;
    }
    print "HTTPD:\n";
    print "  pid:     $$info{'pid'}\n";
    print "  url:     $$info{'url'}\n";
    return 1;
}

1;
